
/*
 * En este servicio se obtiene la posicion actual del terminal a travs de MyLocation
 * y se reporta al servidor con un xmlhttprequest 
 * 
 */

package com.pfc.localiza;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import com.pfc.localiza.MyLocation.LocationResult;
import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.telephony.TelephonyManager;





public class ReportService extends IntentService {
	
	Location ReportLocation;
	
    public ReportService() {
        super("ReportService");      
    	}
    
    
    @Override
    protected void onHandleIntent(Intent intent) {
    
    	/*En primer lugar obtenemos la posicin actual del terminal*/   	
    	
    	LocationResult locationResult = new LocationResult(){
    	    @Override
    	    public void gotLocation(Location location,String locationSource){
    	        //Got the location!  
    	    	if (isNewLocation(location) && isNetworkAvailable())
		  		{   		  			
		  			//Si es una posicion nueva  y hay red, enviamos la nueva posicin		  			
					reportToServer(location,locationSource);      
		  		}
	  		    	    	    	    	
    	    }
    	};
    	MyLocation myLocation = new MyLocation();
    	myLocation.getLocation(this, locationResult);
    	
        //Confirmamos a ReportReceiver la finalizacin
        ReportReceiver.completeWakefulIntent(intent); 
        
    }
    
  public void reportToServer (Location location,String locationSource){
	  
	  	SharedPreferences prefs = getSharedPreferences("Localiza_preferences",Context.MODE_PRIVATE);		
  		final TelephonyManager tm = (TelephonyManager) getBaseContext().getSystemService(Context.TELEPHONY_SERVICE);
  	
  		
  		//tm.getDeviceId() nos da el identificador unico del terminal, normalmente el IMEI
  		//tm.getLine1Number(); nos da el nmero de telfono del terminal. Sin embargo, cuando el nmero no est 
  		//guardado en la SIM, este mtodo no es vlido. Esto ocurre con muchos proveedores, por lo que no es fiable del todo. 
  		//No existe sin embargo otro mecanismo ms eficaz, por lo que utilizamos este.
  		
  		//Formateamos la fecha y hora en la que se ha conseguido la localizacin:
  	   
  	    SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss",Locale.getDefault()); 
  	    String locationtime = formatter.format(new Date(location.getTime()));
  	    
  	     
  		//Informamos los parametros para la peticion POST
        List<NameValuePair> postvalues = new ArrayList<NameValuePair>(10);  
   		postvalues.add(new BasicNameValuePair("imei",tm.getDeviceId()));  
   		postvalues.add(new BasicNameValuePair("phonenumber",tm.getLine1Number()));  
   		postvalues.add(new BasicNameValuePair("lat",Double.toString(location.getLatitude())));
   		postvalues.add(new BasicNameValuePair("lng",Double.toString(location.getLongitude())));
   		postvalues.add(new BasicNameValuePair("alt",Double.toString(location.getAltitude())));
   		postvalues.add(new BasicNameValuePair("spd",Float.toString(location.getSpeed())));
   		postvalues.add(new BasicNameValuePair("acc",Float.toString(location.getAccuracy())));
   		postvalues.add(new BasicNameValuePair("source",locationSource));
  		postvalues.add(new BasicNameValuePair("deviceid", prefs.getString("deviceid", "")));
  		postvalues.add(new BasicNameValuePair("devicekey",prefs.getString("devicekey", "") ));
  		postvalues.add(new BasicNameValuePair("locationtime", locationtime));
   		
   	
   		
   		HttpHandler handler = new HttpHandler();  
		
   		String txt = handler.post("http://www.localiza-app.com/device/Device_location_report.php",(ArrayList<NameValuePair>) postvalues);
   		
   		
   		if (txt.equals("1"))
   		{
   			//Si ha habido exito enviando la localizacin al servidor, se almacena como la ltima posicin enviada
   			SharedPreferences.Editor editor = prefs.edit(); 
			editor.putLong("Prev_lat", Double.doubleToLongBits(location.getLatitude()));
			editor.putLong("Prev_lng", Double.doubleToLongBits(location.getLongitude()));
			editor.putLong("Prev_time", location.getTime());
			editor.commit();
   			
   		}
		
	  
  } 
 
  private boolean isNetworkAvailable() {
	    getApplicationContext();
		ConnectivityManager connectivityManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
	    NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
	    return activeNetworkInfo != null && activeNetworkInfo.isConnected();
	}
  
  
  
  private boolean isNewLocation(Location location)
  {
	  if (location==null)
	  {  
		  return false;
	  }
	  
	  //comparamos con la ltima localizacin remitida al servidor
	  
		SharedPreferences prefs = getSharedPreferences("Localiza_preferences",Context.MODE_PRIVATE);
		
		double Prev_latitude = Double.longBitsToDouble(prefs.getLong("Prev_lat", 0));
		double Prev_longitude = Double.longBitsToDouble(prefs.getLong("Prev_lng", 0));
		long Prev_time = prefs.getLong("Prev_time", 0);
		
		Location Prev_location=new Location(location);
		Prev_location.setLatitude(Prev_latitude);
		Prev_location.setLongitude(Prev_longitude);
	  
	  if (((location.getLatitude()==Prev_latitude) && (location.getLongitude()==Prev_longitude) && (location.getTime()==Prev_time)))
	  {
		  return false;
	  }
	  
	  //analizamos distancia entre ambos puntos para ver si hay mas de 20 metros	  
	  
	  if (Prev_time!=0 && location.distanceTo(Prev_location)<20)
	  {
		  return false;
	  }
	  
	  return true;
	  
  }
  
}



